#include "MovingObject.h"
#include "OgreSceneManager.h"
#include "OgreEntity.h"

MovingObject::MovingObject(std::string modelName, std::string nodeName, Ogre::SceneManager *sm) : 
	mPosition(Ogre::Vector3::ZERO), mName(nodeName),  mVelocity(Ogre::Vector3::ZERO)
{
	mEntity =sm->createEntity(modelName);
	mSceneNode =sm->getRootSceneNode()->createChildSceneNode();
	mSceneNode->attachObject(mEntity);
	mCollision = new OBB(mEntity->getBoundingBox());
}


MovingObject::~MovingObject(void)
{
}

void MovingObject::setScale(Ogre::Vector3 scale)
{
	mSceneNode->setScale(scale);
	mCollision->setScale(scale);

}

void MovingObject::setOrientation(Ogre::Quaternion orientation)
{
	mSceneNode->setOrientation(orientation);
	mCollision->setOrientation(mSceneNode->getOrientation());


}


void  MovingObject::setPosition(Ogre::Vector3 pos) 
{  
	mPosition = pos;
	mSceneNode->setPosition(pos);
	mCollision->setPosition(pos);
}


Ogre::Vector3 MovingObject::getFacing() 
{
	return  mSceneNode->getOrientation() * Ogre::Vector3::UNIT_Z;
}

void MovingObject::translate(Ogre::Vector3 delta)
{
	mSceneNode->translate(delta);
	mPosition = mSceneNode->getPosition();
	mCollision->setPosition(mSceneNode->getPosition());
}



void MovingObject::yaw(Ogre::Degree yawVal)
{
	mSceneNode->yaw(yawVal);
	mCollision->setOrientation(mSceneNode->getOrientation());
}
void MovingObject::pitch(Ogre::Degree pitchVal)
{
	mSceneNode->pitch(pitchVal);
	mCollision->setOrientation(mSceneNode->getOrientation());

}
void MovingObject::roll(Ogre::Degree rollVal)
{
	mSceneNode->roll(rollVal);
	mCollision->setOrientation(mSceneNode->getOrientation());
}




void MovingObject::setVisible(bool isVisible)
{
	mSceneNode->setVisible(isVisible);
}


void MovingObject::setMaterial(Ogre::String materialName)
{
	mEntity->setMaterialName(materialName);

}



