#pragma once
#include "CollisionManager.h"
#include "OgreVector3.h"
#include "OgreQuaternion.h"

namespace Ogre 
{
	class SceneNode;
	class Entity;
	class SceneManager;
}

class MovingObject
{
public:
	MovingObject(std::string modelName, std::string nodeName, Ogre::SceneManager *sm);
	~MovingObject(void);

	Ogre::Vector3 getPosition() { return mPosition; }
	Ogre::Vector3 getVelocity() { return mVelocity; }

	Ogre::Vector3 getFacing();

	void setPosition(Ogre::Vector3 pos);
	void setVelocity(Ogre::Vector3 vel) {  mVelocity = vel; }

	void setOrientation(Ogre::Quaternion orientation);


	void setScale(Ogre::Vector3 scale);

	void translate(Ogre::Vector3 delta);


	void yaw(Ogre::Degree yawVal);
	void pitch(Ogre::Degree pitchVal);
	void roll(Ogre::Degree rollVal);

	void setMaterial(Ogre::String materialName);

	void setVisible(bool isVisible);

	OBB *getOBB() {return mCollision;} 
	
	std::string name() { return mName;}
	Ogre::SceneNode *getSceneNode() { return mSceneNode;}

	
private:
	OBB *mCollision;
	Ogre::SceneNode *mSceneNode;
	Ogre::Entity *mEntity;
	Ogre::Vector3 mPosition;
	Ogre::Vector3 mVelocity;
	std::string mName;
};
